#include "crossroads.h"

void dealloc_reporter (Service *s) {
    int shmid;

    msg ("Freeing reporting memory for service %s (key %d)",
	 s->name, s->shmkey);

    /* Get the shared memory ID. If this fails and we're in stage 0,
     * then ignore the error. The true handler will have already released it.
     */
    if ( (shmid = shmget (s->shmkey, sizeof(Servicereport),
			  0644) ) >= 0)
	msg ("Got existing shm at id %d", shmid);
    else if (program_stage != stage_main)
	    error ("Cannot get shared memory for service %s (key %d: %s)",
		   s->name, s->shmkey, strerror(errno));
    else
	return;

    /* Mark to be released. If this fails and if we're stage 0,
     * then ignore.
     */
    if (shmctl (shmid, IPC_RMID, 0) < 0 && program_stage != stage_main)
	error ("Cannot mark shared memory as destructable: %s "
	       "NOTE: Root will need to 'ipcrm -m %d' and 'ipcrm -s %d' "
	       "to clean up!",
	       strerror(errno), shmid, semid);
    if (semctl (semid, 0, IPC_RMID) < 0 && program_stage != stage_main)
	error ("Cannot remove semaphore: %s "
	       "NOTE: Root will need to 'ipcrm -s %d' to clean up!",
	       semid);
}
